/*----------------------------------------------------------------------- 
 * 
 * File Name: FindChirpChisq.h
 *
 * Author: Anderson, W. G., and Brown, D. A.
 * 
 * Revision: $Id: FindChirpChisq.h,v 1.7 2003/03/31 19:37:23 duncan Exp $
 * 
 *-----------------------------------------------------------------------
 */

#if 0
<lalVerbatim file="FindChirpChisqHV">
Author: Anderson, W. G., and Brown, D. A.
$Id: FindChirpChisq.h,v 1.7 2003/03/31 19:37:23 duncan Exp $
</lalVerbatim> 

<lalLaTeX>
\section{Header \texttt{FindChirpChisq.h}}
\label{s:FindChirpChisq.h}

Provides prototypes and functions to perform a $\chi^2$ veto on binary
inspiral chirps using data generated by the \texttt{FindChirpFilter()}
function.

\subsection*{Synopsis}

\begin{verbatim}
#include <lal/FindChirpChisq.h>
\end{verbatim}

\input{FindChirpChisqHDoc}
</lalLaTeX>
#endif

#ifndef _FINDCHIRPCHISQH_H
#define _FINDCHIRPCHISQH_H

#include <lal/LALDatatypes.h>

#ifdef  __cplusplus
extern "C" {
#pragma }
#endif


NRCSID (SPFINDCHIRPCHISQH, "$Id: FindChirpChisq.h,v 1.7 2003/03/31 19:37:23 duncan Exp $");

#if 0
<lalLaTeX> 
\subsection*{Error codes} 
</lalLaTeX>
#endif
/* <lalErrTable> */
#define FINDCHIRPCHISQH_ENULL 1
#define FINDCHIRPCHISQH_ENNUL 2
#define FINDCHIRPCHISQH_ENUMZ 3
#define FINDCHIRPCHISQH_ECHIZ 4
#define FINDCHIRPCHISQH_EALOC 5
#define FINDCHIRPCHISQH_EMTCH 6
#define FINDCHIRPCHISQH_MSGENULL "Null pointer"
#define FINDCHIRPCHISQH_MSGENNUL "Non-null pointer"
#define FINDCHIRPCHISQH_MSGENUMZ "Number of points is zero or negative"
#define FINDCHIRPCHISQH_MSGECHIZ "Number of chisq bins is zero or negative"
#define FINDCHIRPCHISQH_MSGEALOC "Memory allocation error"
#define FINDCHIRPCHISQH_MSGEMTCH "Invalid template bank match"
/* </lalErrTable> */

#if 0
<lalLaTeX>
\subsection*{Types}
</lalLaTeX>
#endif


/* --- structure for storing input to the chisq veto --------------------- */
/* <lalVerbatim file="FindChirpChisqHFindChirpChisqInput"> */
typedef struct
tagFindChirpChisqInput
{
  COMPLEX8Vector               *qtildeVec;
  COMPLEX8Vector               *qVec;
}
FindChirpChisqInput;
/* </lalVerbatim> */
#if 0
<lalLaTeX>
\subsubsection*{Structure \texttt{FindChirpChisqInput}}
\idx[Type]{FindChirpChisqInput}

\input{FindChirpChisqHFindChirpChisqInput}

\noindent This structure contains the input to the $\chi^2$ veto function.
The quantities should be populated by \texttt{FindChirpFilter()} on entry.
The fields are:

\begin{description}
\item[\texttt{COMPLEX8Vector *qtildeVec}] A vector containing the frequncy
domain quantity $\tilde{q}_k$ as defined in \texttt{FindChirpFilter()}.

\item[\texttt{COMPLEX8Vector *qVec}] A vector containing the time
domain quantity $q_j$ as defined in \texttt{FindChirpFilter()}.
\end{description}
</lalLaTeX>
#endif

/* --- parameter structure for the chisq veto ---------------------------- */
/* <lalVerbatim file="FindChirpChisqHFindChirpChisqParams"> */
typedef struct
tagFindChirpChisqParams
{
  REAL4                         norm;
  REAL4                         bankMatch;
  UINT4Vector                  *chisqBinVec;
  ComplexFFTPlan               *plan;
  COMPLEX8Vector               *qtildeBinVec;
  COMPLEX8Vector              **qBinVecPtr;
}
FindChirpChisqParams;
/* </lalVerbatim> */
#if 0
<lalLaTeX>
\subsubsection*{Structure \texttt{FindChirpChisqParams}}
\idx[Type]{FindChirpChisqParams}

\input{FindChirpChisqHFindChirpChisqParams}

\noindent This structure contains the parameters used by the $\chi^2$ veto
function \texttt{FindChirpChisqVeto()}.  It is created and destroyed by the
\texttt{FindChirpChisqVetoInit()} and \texttt{FindChirpChisqVetoFinalize()}
functions. The fields are:

\begin{description}
\item[\texttt{REAL4 norm}] The normalization...

\item[\texttt{REAL4 bankMatch}] Template bank match...

\item[\texttt{UINT4Vector *chisqBinVec}] A vector containing the...

\item[\texttt{ComplexFFTPlan *plan}] The FFTW plan used by the inverse DFT.

\item[\texttt{COMPLEX8Vector *qtildeBinVec}] ...

\item[\texttt{COMPLEX8Vector **qBinVecPtr}] ...
\end{description}
</lalLaTeX>
#endif

#if 0
<lalLaTeX>
\vfill{\footnotesize\input{FindChirpChisqHV}}
</lalLaTeX> 
#endif

#if 0
<lalLaTeX>
\newpage\input{FindChirpChisqC}
</lalLaTeX>
#endif

void
LALFindChirpChisqVetoInit (
    LALStatus                  *status,
    FindChirpChisqParams       *params,
    UINT4                       numChisqBins,
    UINT4                       numPoints
    );

void
LALFindChirpChisqVetoFinalize (
    LALStatus                  *status,
    FindChirpChisqParams       *params,
    UINT4                       numChisqBins
    );

void
LALFindChirpChisqVeto (
    LALStatus                  *status,
    REAL4Vector                *chisqVec,
    FindChirpChisqInput        *input,
    FindChirpChisqParams       *params
    );

#ifdef  __cplusplus
#pragma {
}
#endif

#endif /* _FINDCHIRPCHISQH_H */
